/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "user_callbacks.h"

#include <stdlib.h>
#include <stdio.h>

#include <player.h>
#include <app_common.h>

#define BUFLEN 256

static Evas_Object *player_start_bt = NULL;
static Evas_Object *player_pause_bt = NULL;
static Evas_Object *player_multiplay_bt = NULL;
static Evas_Object *player_display = NULL;
static player_h g_player = NULL;
static player_h player_1 = NULL;
static player_h player_2 = NULL;
static player_h player_3 = NULL;
static const char *player_resource_file = "sample.3gp";
static const char *multiplayer_resource_file1 = "test0.3gp";
static const char *multiplayer_resource_file2 = "test1.wav";
static const char *multiplayer_resource_file3 = "test2.wav";

// Callback function invoked when the Player playback process is finished.
static void _player_completed_cb(void *data)
{
    PRINT_MSG("Playback finished.");
}

// Callback function invoked when the player playback process was interrupted.
static void _player_interrupted_cb(player_interrupted_code_e code, void *data)
{
    PRINT_MSG("Player playback process was interrupted.");

    // Check the current state of the player.
    player_state_e state;

    int error_code = player_get_state(g_player, &state);
    if (PLAYER_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("player_get_state()", error_code);
        return;
    }

    if (PLAYER_INTERRUPTED_COMPLETED == code) {
        if (PLAYER_STATE_PAUSED == state) {
            // If the player interruption is finished and the player is currently paused, then resume the playback.
            error_code = player_start(g_player);
            if (PLAYER_ERROR_NONE != error_code) {
                DLOG_PRINT_ERROR("player_start()", error_code);
            } else
                elm_object_text_set(player_pause_bt, "Pause video");
        }
    } else {
        if (PLAYER_STATE_PLAYING == state) {
            // If the player is currently playing, pause the playback because of the incoming interruption.
            error_code = player_pause(g_player);
            if (PLAYER_ERROR_NONE != error_code) {
                DLOG_PRINT_ERROR("player_pause()", error_code);
            } else
                elm_object_text_set(player_pause_bt, "Resume video");
        }
    }
}

// Callback function invoked when the third player finished its playback.
static void _player3_completed_cb(void *user_data)
{
    PRINT_MSG("Player 3 finished.");
}

// Callback function invoked when the second player finished its playback.
static void _player2_completed_cb(void *user_data)
{
    PRINT_MSG("Player 2 finished.");
}

// Callback function invoked when the first player finished its playback.
static void _player1_completed_cb(void *user_data)
{
    PRINT_MSG("Player 1 finished.");
}

// Callback invoked when the "Play multiple sounds" button is clicked.
static void __player_cb_multiplay(appdata_s *ad, Evas_Object *obj, void *event_info)
{
    // Create a handle for manipulating the first player.
    int error_code = player_create(&player_1);
    if (PLAYER_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("player_create()", error_code);
        return;
    }

    // Create a handle for manipulating the second player.
    error_code = player_create(&player_2);
    if (PLAYER_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("player_create()", error_code);
        return;
    }

    // Create a handle for manipulating the third player.
    error_code = player_create(&player_3);
    if (PLAYER_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("player_create()", error_code);
        return;
    }

    // Set audio files that will be played by all three players:
    // 1. Get the path to the resources directory.
    char *shared_path = app_get_resource_path();

    // 2. Create the full path to the file that will be played by the first player.
    char path[BUFLEN];
    snprintf(path, BUFLEN, "%s%s", shared_path, multiplayer_resource_file1);
    PRINT_MSG("File: %s", path);

    // 3. Set the path to file used by the first player.
    error_code = player_set_uri(player_1, path);
    if (PLAYER_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("player_set_uri()", error_code);
        free(shared_path);
        return;
    }

    // 2. Create the full path to the file that will be played by the second player.
    snprintf(path, BUFLEN, "%s%s", shared_path, multiplayer_resource_file2);
    PRINT_MSG("File: %s", path);

    // 3. Set the path to file used by the second player.
    error_code = player_set_uri(player_2, path);
    if (PLAYER_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("player_set_uri()", error_code);
        free(shared_path);
        return;
    }

    // 2. Create the full path to the file that will be played by the third player.
    snprintf(path, BUFLEN, "%s%s", shared_path, multiplayer_resource_file3);
    free(shared_path);
    PRINT_MSG("File: %s", path);

    // 3. Set the path to file used by the third player.
    error_code = player_set_uri(player_3, path);
    if (PLAYER_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("player_set_uri()", error_code);
        return;
    }

    // Set callback function that will be invoked when the first player finishes its playback.
    error_code = player_set_completed_cb(player_1, _player1_completed_cb, NULL);
    if (PLAYER_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("player_set_completed_cb()", error_code);
        return;
    }

    // Set callback function that will be invoked when the second player finishes its playback.
    error_code = player_set_completed_cb(player_2, _player2_completed_cb, NULL);
    if (PLAYER_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("player_set_completed_cb()", error_code);
        return;
    }

    // Set callback function that will be invoked when the third player finishes its playback.
    error_code = player_set_completed_cb(player_3, _player3_completed_cb, NULL);
    if (PLAYER_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("player_set_completed_cb()", error_code);
        return;
    }

    // Prepare the first player for the playback.
    error_code = player_prepare(player_1);
    if (PLAYER_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("player_prepare()", error_code);
        return;
    }

    // Prepare the second player for the playback.
    error_code = player_prepare(player_2);
    if (PLAYER_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("player_prepare()", error_code);
        return;
    }

    // Prepare the third player for the playback.
    error_code = player_prepare(player_3);
    if (PLAYER_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("player_prepare()", error_code);
        return;
    }

    // Start the first player playback process.
    error_code = player_start(player_1);
    if (PLAYER_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("player_start()", error_code);
        return;
    }

    // Start the second player playback process.
    error_code = player_start(player_2);
    if (PLAYER_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("player_start()", error_code);
        return;
    }

    // Start the third player playback process.
    error_code = player_start(player_3);
    if (PLAYER_ERROR_NONE != error_code)
        DLOG_PRINT_ERROR("player_start()", error_code);
}

// Callback invoked when the "Pause" button is clicked.
static void __player_cb_pause(appdata_s *ad, Evas_Object *obj, void *event_info)
{
    // Check the current state of the player.
    player_state_e state;

    int error_code = player_get_state(g_player, &state);
    if (PLAYER_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("player_get_state()", error_code);
        return;
    }

    if (PLAYER_STATE_PAUSED == state) {
        // Resume the video.
        error_code = player_start(g_player);
        if (PLAYER_ERROR_NONE != error_code) {
            DLOG_PRINT_ERROR("player_start()", error_code);
            return;
        }

        PRINT_MSG("Playback resumed.");
        elm_object_text_set(player_start_bt, "Stop video");
        elm_object_text_set(player_pause_bt, "Pause video");
    } else if (PLAYER_STATE_PLAYING == state) {
        // Pause the playback.
        error_code = player_pause(g_player);
        if (PLAYER_ERROR_NONE != error_code)
            DLOG_PRINT_ERROR("player_pause()", error_code);
        else
            elm_object_text_set(player_pause_bt, "Resume video");
    }
}

// Callback invoked when the "Play video" button is clicked.
static void __player_cb_play(appdata_s *ad, Evas_Object *obj, void *event_info)
{
    // Check the current state of the player.
    player_state_e state;

    int error_code = player_get_state(g_player, &state);
    if (PLAYER_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("player_get_state()", error_code);
        return;
    }

    if (PLAYER_STATE_READY == state) {
        // Start playing the video.
        error_code = player_start(g_player);
        if (PLAYER_ERROR_NONE != error_code) {
            DLOG_PRINT_ERROR("player_start()", error_code);
            return;
        }

        PRINT_MSG("Playback started.");
        elm_object_text_set(player_start_bt, "Stop video");

        // Enable the "Pause" button when the video is being played.
        elm_object_disabled_set(player_pause_bt, EINA_FALSE);
    } else if (PLAYER_STATE_PLAYING == state || PLAYER_STATE_PAUSED == state) {
        // Stop the playback process.
        error_code = player_stop(g_player);
        if (PLAYER_ERROR_NONE != error_code) {
            DLOG_PRINT_ERROR("player_stop()", error_code);
            return;
        }

        PRINT_MSG("Playback stopped.");
        evas_object_hide(player_display);
        elm_object_text_set(player_start_bt, "Play video");
        elm_object_text_set(player_pause_bt, "Pause video");

        // Disable the "Pause" button when no video is being played.
        elm_object_disabled_set(player_pause_bt, EINA_TRUE);
    }
}

// Callback invoked when the "Player" screen is being closed.
void player_pop_cb()
{
    // Hide the player display (UI element).
    evas_object_hide(player_display);

    // Stop playing the video.
    player_stop(g_player);

    // Reset the player handle.
    player_unprepare(g_player);

    // Release the memory allocated for the player instance.
    player_destroy(g_player);
    g_player = NULL;

    // Release the resources allocated for multiple playback:
    player_stop(player_1);
    player_unprepare(player_1);
    player_destroy(player_1);
    player_1 = NULL;

    player_stop(player_2);
    player_unprepare(player_2);
    player_destroy(player_2);
    player_2 = NULL;

    player_stop(player_3);
    player_unprepare(player_3);
    player_destroy(player_3);
    player_3 = NULL;
}

static void _post_render_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
    // Get the size of the parent box.
    int x = 0, y = 0, w = 0, h = 0;
    evas_object_geometry_get(obj, &x, &y, &w, &h);

    // Set the size of the image object.
    evas_object_resize(player_display, w, h);
    evas_object_move(player_display, 0, y);
}

// Player menu creation.
void create_buttons_in_main_window(appdata_s *ad)
{
    // Create the window for the Player.
    Evas_Object *display = _create_new_cd_display(ad, "Player", NULL);

    // Create buttons for the Player.
    player_start_bt = _new_button(ad, display, "Play video", __player_cb_play);
    player_pause_bt = _new_button(ad, display, "Pause video", __player_cb_pause);
    player_multiplay_bt = _new_button(ad, display, "Play multiple sounds", __player_cb_multiplay);

    // Disable the "Pause" button when no video is being played.
    elm_object_disabled_set(player_pause_bt, EINA_TRUE);

    // Create image UI object. It will be used as a display for the video file.
    Evas_Object *video_display = evas_object_data_get(display, "video");
    Evas *evas = evas_object_evas_get(video_display);
    player_display = evas_object_image_add(evas);
    evas_object_event_callback_add(video_display, EVAS_CALLBACK_RESIZE, _post_render_cb, NULL);

    // Create a handle to the player.
    int error_code = player_create(&g_player);
    if (PLAYER_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("player_create()", error_code);
        return;
    }

    // Set the callback function that will be called when something will interrupt the playback process.
    error_code = player_set_interrupted_cb(g_player, _player_interrupted_cb, NULL);
    if (PLAYER_ERROR_NONE != error_code)
        DLOG_PRINT_ERROR("player_set_interrupted_cb()", error_code);

    // Set the callback function that will be called when the playback process is finished.
    error_code = player_set_completed_cb(g_player, _player_completed_cb, NULL);
    if (PLAYER_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("player_set_completed_cb()", error_code);
        return;
    }

    // Get the path to the file which will be used for playback.
    char *shared_path = app_get_resource_path();
    char path[BUFLEN];
    snprintf(path, BUFLEN, "%s%s", shared_path, player_resource_file);
    free(shared_path);

    // Set the obtained file path as a source for the Player.
    error_code = player_set_uri(g_player, path);
    if (PLAYER_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("player_set_uri()", error_code);
        return;
    }

    PRINT_MSG("%s will be played.", path);

    // Set the display for the video.
    error_code =
        player_set_display(g_player, PLAYER_DISPLAY_TYPE_EVAS, GET_DISPLAY(player_display));
    if (PLAYER_ERROR_NONE != error_code) {
        DLOG_PRINT_ERROR("player_set_display()", error_code);
        return;
    }

    // Prepares the Player for the playback.
    error_code = player_prepare(g_player);
    if (PLAYER_ERROR_NONE != error_code)
        DLOG_PRINT_ERROR("player_prepare()", error_code);
}
